#ifndef _CREATETRIGGERS_CPP
#define _CREATETRIGGERS_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <SQL.H>
#include <SQLExt.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"
#include "../../SharedClasses/CRC32/CRC.H"
#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../CSockSrvr/CSockSrvr.H"

#include "NSWFL.H"
#include "Init.H"
#include "Entry.H"
#include "Routines.H"
#include "Console.H"
#include "CreateTriggers.H"
#include "GetPKs.H"

#include "../Dialogs/ReplicationDlg.H"
#include "../Dialogs/MainDlg.H"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//This function is complete!
bool DropReplicationTriggers(CSQL *lpCSQL, char *sDB, char *sTable)
{
	char sSQL[1024];

	sprintf(sSQL, "DROP TRIGGER [SQLExch_%s_%s_Insert_Trigger]", sDB, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TRIGGER [SQLExch_%s_%s_Update_Trigger]", sDB, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TRIGGER [SQLExch_%s_%s_Delete_Trigger]", sDB, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//This function is complete!
bool DropReplicationTables(CSQL *lpCSQL, char *sTrgDB, char *sDB, char *sDBO, char *sTable)
{
	char sSQL[1024];

	sprintf(sSQL, "DROP TABLE [%s].[%s].[SQLExch_%s_%s_INS]", sTrgDB, sDBO, sDB, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TABLE [%s].[%s].[SQLExch_%s_%s_UPD]", sTrgDB, sDBO, sDB, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	sprintf(sSQL, "DROP TABLE [%s].[%s].[SQLExch_%s_%s_DEL]", sTrgDB, sDBO, sDB, sTable);
	lpCSQL->ExecuteNonQuery(sSQL);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//This function is complete!
bool CreateReplicationTables(CSQL *lpCSQL, char *sTrgDB, char *sDB, char *sDBO, char *sTable)
{
	char sSmlTmp[64];
	char sLargeBaseSQL[10240];
	char sLargeSQL[10240];

	PRIMARY_KEYS MyPKs;

	if(GetPrimaryKeysEx(lpCSQL, sTable, &MyPKs))
	{
		int iKey = 0;

		sprintf(sLargeBaseSQL, "CREATE TABLE [%s].[%s].[SQLExch_%s_%s_%%s] (\r\n", sTrgDB, sDBO, sDB, sTable);

		while(iKey < MyPKs.iPKs)
		{
			strcat(sLargeBaseSQL, "\t[");
			strcat(sLargeBaseSQL, MyPKs.sPKs[iKey]);
			strcat(sLargeBaseSQL, "] ");

			strcat(sLargeBaseSQL, MyPKs.sPKType[iKey]);

			if(MyPKs.iPKStatus[iKey] != 0)
			{
				sprintf(sSmlTmp, " (%d)", MyPKs.iPKLen[iKey]);
				strcat(sLargeBaseSQL, sSmlTmp);
			}

			strcat(sLargeBaseSQL, " NULL,\r\n");

			iKey++;
		}

		//-------------------------------------------------------------
		//Need to add a SQLExch_Pending Bit column to each of the new tables.
		//	We use this column because of the following scenario:
		//		
		//		1) When a transfer begins we need to update SQLExch_Pending = 1.
		//		2) Next we need to select all of the rows where SQLExch_Pending = 1.
		//		3) While the Server and client excange data, and do imports and exports,
		//			the data may have changed, adding more rows to the table with a
		//			SQLExch_Pending flag equal to 0
		//		4) If the impot was a success then we delete from the table where SQLExch_Pending = 1
		//		5) The rows that were added durring the server/client conversation are
		//			preserved for the next transfer
		//-------------------------------------------------------------
		strcat(sLargeBaseSQL, "\t[SQLExch_Pending] Bit\r\n");
		strcat(sLargeBaseSQL, ") ON [PRIMARY]\r\n");

		sprintf(sLargeSQL, sLargeBaseSQL, "INS");
		lpCSQL->ExecuteNonQuery(sLargeSQL);

		sprintf(sLargeSQL, sLargeBaseSQL, "UPD");
		lpCSQL->ExecuteNonQuery(sLargeSQL);

		sprintf(sLargeSQL, sLargeBaseSQL, "DEL");
		lpCSQL->ExecuteNonQuery(sLargeSQL);

		FreePrimaryKeysEx(&MyPKs);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//This function is complete!
bool CreateReplicationTriggers(CSQL *lpCSQL, char *sTrgDB, char *sDB, char *sDBO, char *sTable)
{
	char sLargeBaseSQL[10240];
	char sLargeSQL[10240];

	PRIMARY_KEYS MyPKs;

	if(GetPrimaryKeysEx(lpCSQL, sTable, &MyPKs))
	{
		int iKey = 0;

		strcpy(sLargeSQL, "");
		while(iKey < MyPKs.iPKs)
		{
			strcat(sLargeSQL, "[");
			strcat(sLargeSQL, MyPKs.sPKs[iKey]);
			strcat(sLargeSQL, "] ");

			if(iKey != (MyPKs.iPKs - 1))
			{
				strcat(sLargeSQL, ",");
			}

			iKey++;
		}

		FreePrimaryKeysEx(&MyPKs);

		sprintf(sLargeBaseSQL, "CREATE TRIGGER [SQLExch_%s_%s_%%s_Trigger] ON [%s].[%s]\r\n"
			"\tFOR %%s\r\n"
			"\tAS\r\n"
			"\t\tINSERT INTO [%s].[%s].[SQLExch_%s_%s_%%s] (%s)\r\n\t\t\tSELECT %s FROM %%s\r\n",
			sDB, sTable, sDBO, sTable,
			sTrgDB, sDBO, sDB, sTable, sLargeSQL, sLargeSQL);

		sprintf(sLargeSQL, sLargeBaseSQL, "Insert", "INSERT", "INS", "Inserted");
		lpCSQL->ExecuteNonQuery(sLargeSQL);

		sprintf(sLargeSQL, sLargeBaseSQL, "Update", "UPDATE", "UPD", "Inserted");
		lpCSQL->ExecuteNonQuery(sLargeSQL);

		sprintf(sLargeSQL, sLargeBaseSQL, "Delete", "DELETE", "DEL", "Deleted");
		lpCSQL->ExecuteNonQuery(sLargeSQL);
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//This function is complete!
bool CreateReplication(CSQL *lpCSQL, char *sTrgDB, char *sDB, char *sDBO, char *sTable)
{
	DropReplicationTriggers(lpCSQL, sDB, sTable);
	DropReplicationTables(lpCSQL, sTrgDB, sDB, sDBO, sTable);

	if(CreateReplicationTables(lpCSQL, sTrgDB, sDB, sDBO, sTable))
	{
		CreateReplicationTriggers(lpCSQL, sTrgDB, sDB, sDBO, sTable);
	}
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool GenerateReplicationScripts(CSQL *lpCSQL, char *sTrgDB, char *sDB, char *sDBO)
{
	/* (Future Selection Model)

		SELECT Address.*
		FROM Address, [SQLExch_Replication].[DBO].[SQLExch_JT_Dellinger_Address_INS]
		WHERE
		[Address].ID = [SQLExch_Replication].[DBO].[SQLExch_JT_Dellinger_Address_INS].[ID] AND
		[Address].Center_Key = [SQLExch_Replication].[DBO].[SQLExch_JT_Dellinger_Address_INS].[Center_Key] AND
		[SQLExch_Replication].[DBO].[SQLExch_JT_Dellinger_Address_INS].[SQLExch_Pending] = 1

	*/

	char sSQL[10240];
	int iTempSz = 0;
	int iLen = 0;
	long lSeq = 1000;

	char sAction[100];

	FILE *hSource = NULL;

	char sOnSuccess[1024];
	char sOnFailure[1024];

	char sLargeBaseSQL[10240];
	char sLargeSQL[10240];
	char sTable[1024];
	char sTemp[1024];

	PRIMARY_KEYS MyPKs;

	CRecordSet rsTables;
	//CRecordSet rsColumns;

	//----------------------------------------------------------------------------------------------------
	//Create the "SQLExch_Statements" table
	//----------------------------------------------------------------------------------------------------
	sprintf(sLargeSQL, "%s\\SQL\\SQLExch_Statements.sql", gsPath);
	if((hSource = fopen(sLargeSQL, "rb")) == NULL)
	{
		MessageBox(MainDialog_hWnd, "Failed to open the SQLExch_Statements file.", gsTitleCaption, 0);
		return false;
	}

	iLen = Get_FileSize(sLargeSQL);
	fread(sLargeSQL, iLen, sizeof(char), hSource);
	sLargeSQL[iLen] = '\0';

	fclose(hSource);

	//Recreate the SQLExch_Statements table.
	lpCSQL->ExecuteNonQuery("DROP TABLE SQLExch_Statements"); 
	
	//Recreate the SQLExch_Statements table.
	lpCSQL->ExecuteNonQuery(sLargeSQL);

	//----------------------------------------------------------------------------------------------------

	strcpy(sSQL, "SELECT Name"
		" FROM SysObjects"
		" WHERE xType = 'U' AND (LEFT(NAME, 8) <> 'SQLExch_'"
		" AND ID IN (SELECT Parent_Obj FROM SysObjects AS A WHERE A.xType = 'TR' AND LEFT(A.NAME, 8) = 'SQLExch_'))"
		//" OR Name = 'SQLExch_Statements'"
		" ORDER BY Name");
	lpCSQL->Execute(sSQL, &rsTables);

	//Loop through all of the tables that have triggers.
	while(rsTables.Fetch())
	{
		rsTables.sColumnEx(1, sTable, sizeof(sTable), &iTempSz);

		if(GetPrimaryKeysEx(lpCSQL, sTable, &MyPKs))
		{
			int iTrgTable = 0;

			while(iTrgTable < 3)
			{
				int iKey = 0;

				strcpy(sLargeBaseSQL, "SELECT %s.*\r\n");
				strcat(sLargeBaseSQL, " FROM %s, [%s].[%s].[SQLExch_%s_%s_%s]\r\n");
				strcat(sLargeBaseSQL, " WHERE \r\n");

				//Loop through all of the primary keys
				while(iKey < MyPKs.iPKs)
				{
					strcat(sLargeBaseSQL, "[");
					strcat(sLargeBaseSQL, sTable);
					strcat(sLargeBaseSQL, "].");

					strcat(sLargeBaseSQL, "[");
					strcat(sLargeBaseSQL, MyPKs.sPKs[iKey]);
					strcat(sLargeBaseSQL, "] = ");

					strcat(sLargeBaseSQL, "[");
					strcat(sLargeBaseSQL, sTrgDB);
					strcat(sLargeBaseSQL, "].");

					strcat(sLargeBaseSQL, "[");
					strcat(sLargeBaseSQL, sDBO);
					strcat(sLargeBaseSQL, "].");

					if(iTrgTable == 0)      strcpy(sAction, "INS");
					else if(iTrgTable == 1) strcpy(sAction, "UPD");
					else if(iTrgTable == 2) strcpy(sAction, "DEL");
														
					strcat(sLargeBaseSQL, "[SQLExch_");
					strcat(sLargeBaseSQL, sDB);
					strcat(sLargeBaseSQL, "_");
					strcat(sLargeBaseSQL, sTable);
					strcat(sLargeBaseSQL, "_");
					strcat(sLargeBaseSQL, sAction);
					strcat(sLargeBaseSQL, "].");

					strcat(sLargeBaseSQL, "[");
					strcat(sLargeBaseSQL, MyPKs.sPKs[iKey]);
					strcat(sLargeBaseSQL, "] AND ");

					iKey++;
				}

				if(iTrgTable == 0)      strcpy(sAction, "INS");
				else if(iTrgTable == 1) strcpy(sAction, "UPD");
				else if(iTrgTable == 2) strcpy(sAction, "DEL");
				
				//----------------------------------------------------------------------------------------
				//Default OnFailure SQL.
				sprintf(sOnSuccess, "DELETE FROM [%s].[%s].[SQLExch_%s_%s_%s] WHERE SQLExch_Pending = 1",
					sTrgDB, sDBO, sDB, sTable, sAction);

				//No default OnFailure SQL.
				strcpy(sOnFailure, "");
				//----------------------------------------------------------------------------------------

				sprintf(sTemp, "UPDATE [%s].[%s].[SQLExch_%s_%s_%s] SET SQLExch_Pending = 1",
					sTrgDB, sDBO, sDB, sTable, sAction);

				sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
					" Sequence, Active, Comments, Identifier, Dirty)"
					" VALUES('%s', '%s', '%s', '%s', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
					sTemp, "NULL", "NULL", "NULL", lSeq, lSeq);
				lpCSQL->ExecuteNonQuery(sSQL);
				lSeq = (lSeq + 10);

				//----------------------------------------------------------------------------------------

				strcat(sLargeBaseSQL, "[%s].[%s].[SQLExch_%s_%s_%s].SQLExch_Pending = 1");
				sprintf(sLargeSQL, sLargeBaseSQL, sTable, sTable, sTrgDB, sDBO, sDB, sTable, sAction,
					sTrgDB, sDBO, sDB, sTable, sAction);

				if(iTrgTable == 0)      strcpy(sAction, "Insert");
				else if(iTrgTable == 1) strcpy(sAction, "Update");
				else if(iTrgTable == 2) strcpy(sAction, "Delete");

				if(iTrgTable == 2)
				{
					strcat(sTable, "_SQLExch_Delete");
				}
				
				sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
					" Sequence, Active, Comments, Identifier, Dirty)"
					" VALUES('%s', '%s', '%s', '%s', '%d', '1', 'Auto Generated.', '%s', 1)",
					sLargeSQL, sOnSuccess, sOnFailure, sTable, lSeq, sAction);
				lpCSQL->ExecuteNonQuery(sSQL);
				lSeq = (lSeq + 10);

				iTrgTable++;
			}

			FreePrimaryKeysEx(&MyPKs);
		}
	}

	rsTables.Close();

	/*
	if(GetPrimaryKeysEx(lpCSQL, sTable, &MyPKs))
	{
		int iKey = 0;

		strcpy(sLargeSQL, "");
		while(iKey < MyPKs.iPKs)
		{
			strcat(sLargeSQL, "[");
			strcat(sLargeSQL, MyPKs.sPKs[iKey]);
			strcat(sLargeSQL, "] ");

			if(iKey != (MyPKs.iPKs - 1))
			{
				strcat(sLargeSQL, ",");
			}

			iKey++;
		}

		FreePrimaryKeysEx(&MyPKs);
	}
*/

	/*
	char **sPKs = NULL;
	int iPKs = 0;

	char sColList[10240];
	char sSQL[10240];
	char sTemp[10240];
	char sTable[255];
	int iTempSz = 0;
	int iColType = 0;
	int iLen = 0;
	long lSeq = 1000;

	strcpy(sSQL, "SELECT Name"
		" FROM SysObjects"
		" WHERE xType = 'U' AND (LEFT(NAME, 8) <> 'SQLExch_'"
		" AND ID IN (SELECT Parent_Obj FROM SysObjects AS A WHERE A.xType = 'TR' AND LEFT(A.NAME, 8) = 'SQLExch_'))"
		//" OR Name = 'SQLExch_Statements'"
		" ORDER BY Name");
	lpCSQL->Execute(sSQL, &rsTables);

	lpCSQL->ExecuteNonQuery("DELETE FROM SQLExch_Statements");

	while(rsTables.Fetch())
	{
		strcpy(sColList, "");

		rsTables.sColumnEx(1, sTable, sizeof(sTable), &iTempSz);

		sprintf(sSQL, "SELECT Name, xType FROM SysColumns WHERE ID ="
			" (SELECT ID FROM SysObjects WHERE Name = '%s')"
			" ORDER BY ColOrder", sTable);
		lpCSQL->Execute(sSQL, &rsColumns);

		while(rsColumns.Fetch())
		{
			rsColumns.sColumnEx(1, sTemp, sizeof(sTemp), &iTempSz);
			iColType = rsColumns.lColumn(2);

			if(iColType == COL_TYPE_TEXT || iColType == COL_TYPE_NTEXT || iColType == COL_TYPE_IMAGE)
			{
				//strcat(sColList, "CONVERT(VARCHAR,");
				//strcat(sColList, sTemp);
				//strcat(sColList, "),");
			}
			else {
				strcat(sColList, "[");
				strcat(sColList, sTemp);
				strcat(sColList, "],");
			}
		}

		if((iLen = strlen(sColList)))
		{
			if(sColList[iLen - 1] == ',')
			{
				sColList[iLen - 1] = '\0';
			}
		}

		//--------------------------------------------------------------------------------------------------------
		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('UPDATE SQLExch_%s_INS SET SQLExch_Pending = 1', NULL, NULL, '', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
			sTable, lSeq, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		strcpy(sTemp, "SELECT ");
		strcat(sTemp, sColList);
		strcat(sTemp, " FROM ");
		strcat(sTemp, " SQLExch_");
		strcat(sTemp, sTable);
		strcat(sTemp, "_INS WHERE SQLExch_Pending = 1");

		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('%s', 'DELETE FROM SQLExch_%s_INS WHERE SQLExch_Pending = 1', NULL, '%s', '%d', '1', 'Auto Generated.', 'Insert', 1)",
			sTemp, sTable, sTable, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		//--------------------------------------------------------------------------------------------------------
		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('UPDATE SQLExch_%s_UPD SET SQLExch_Pending = 1', NULL, NULL, '', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
			sTable, lSeq, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		strcpy(sTemp, "SELECT ");
		strcat(sTemp, sColList);
		strcat(sTemp, " FROM ");
		strcat(sTemp, " SQLExch_");
		strcat(sTemp, sTable);
		strcat(sTemp, "_UPD WHERE SQLExch_Pending = 1");

		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('%s', 'DELETE FROM SQLExch_%s_UPD WHERE SQLExch_Pending = 1', NULL, '%s', '%d', '1', 'Auto Generated.', 'Update', 1)",
			sTemp, sTable, sTable, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		//--------------------------------------------------------------------------------------------------------
		sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
			" Sequence, Active, Comments, Identifier, Dirty)"
			" VALUES('UPDATE SQLExch_%s_DEL SET SQLExch_Pending = 1', NULL, NULL, '', '%d', '1', 'Auto Generated.', 'SQLExch_Pending_Update_%d', 1)",
			sTable, lSeq, lSeq);
		lpCSQL->ExecuteNonQuery(sSQL);
		lSeq = (lSeq + 10);

		//Get a list of primary keys for the import table.
		sPKs = GetPrimaryKeys(lpCSQL, sTable, &iPKs);
		if(iPKs > 0)
		{
			int iPK = 0;

			//---------------------------------
			//Make a list of the primary keys.
			//---------------------------------
			strcpy(sColList, "");
			while(iPK < iPKs)
			{
				strcat(sColList, "[");
				strcat(sColList, sPKs[iPK]);
				strcat(sColList, "],");
				iPK++;
			}

			if((iLen = strlen(sColList)))
			{
				if(sColList[iLen - 1] == ',')
				{
					sColList[iLen - 1] = '\0';
				}
			}
			//---------------------------------

			strcpy(sTemp, "SELECT ");
			strcat(sTemp, sColList);
			strcat(sTemp, " FROM ");
			strcat(sTemp, " SQLExch_");
			strcat(sTemp, sTable);
			strcat(sTemp, "_DEL WHERE SQLExch_Pending = 1");

			sprintf(sSQL, "INSERT INTO SQLExch_Statements (Statement, OnSuccess, OnFailure, ImportTable,"
				" Sequence, Active, Comments, Identifier, Dirty)"
				" VALUES('%s', 'DELETE FROM SQLExch_%s_DEL WHERE SQLExch_Pending = 1', NULL, '%s_SQLExch_Delete', '%d', '1', 'Auto Generated.', 'Delete', 1)",
				sTemp, sTable, sTable, lSeq);
			lpCSQL->ExecuteNonQuery(sSQL);
			lSeq = (lSeq + 10);
		}

		FreePrimaryKeys(sPKs, iPKs);

		rsColumns.Close();
	}		

	rsTables.Close();
*/
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
